from typing import List
from typing_extensions import override

from .agentic_rag import AgenticRAG
from ..config.agentic_rag_config import AgenticRAGConfig
from ..vectorestore.vectorStore import VectorStore
from ..scrapper.github_scrapper import GithubScrapper
from ..config.agentic_rag_config import SimpleAgenticRAGConfig
from .builder import Builder
from ..config.settings import Settings
from ..models.data_source_model import DataSource
from .simple_rag_api import RAGPipeline


class AgenticRAGPipeline(RAGPipeline):
    def __init__(
        self,
        knowledge_base: List[DataSource],
        config: SimpleAgenticRAGConfig,
    ) -> None:
        """
        Initializes the AgenticRAGPipeline with a knowledge base and model.

        Args:
            knowledge_base (List[DataSource]): A list of data source objects (e.g., FolderSource, GitHubSource).
            k (int, optional): The number of top documents to retrieve. Defaults to 5.
            model_name (str, optional): The name of the LLM to use. Defaults to Settings.DEFAULT_LLM.
            provider (str, optional): The name of the LLM provider you want to use : Ollama.
        """
        self.knowledge_base: List[DataSource] = knowledge_base
        model_embeddings: str = Settings.DEFAULT_EMBEDDINGS_MODEL
        persist_directory: str = Settings.DEFAULT_PERSIST_DIRECTORY
        collection_name: str = Settings.DEFAULT_COLLECTION_NAME
        self.file_extension: str = Settings.DEFAULT_EXTENSIONS

        self.vector_store = (
            Builder()
            .with_embeddings(Settings.HUGGINGFACE, model_name=model_embeddings)
            .with_vector_store(
                Settings.CHROMA,
                persist_directory=persist_directory,
                collection_name=collection_name,
            )
            .build_vector_store()
        )

        config = AgenticRAGConfig(
            vector_store=self.vector_store,
            api_key=config.api_key,
            provider=config.provider,
            model=config.model,
            num_ctx=config.num_ctx,
            k=config.k,
            verbosity_level=config.verbosity_level,
            max_steps=config.max_steps,
            system_prompt=config.system_prompt,
        )
        self.agenticRag = AgenticRAG(config)

        self.github_scrapper: GithubScrapper = GithubScrapper()

    @override
    def get_vector_store(self) -> VectorStore:
        return self.vector_store

    @override
    def generate(self, question: str, stream: bool = False) -> str:
        """
        Asks a question to the pipeline and retrieves the generated answer.

        Args:
            question (str): The question to ask the pipeline.

        Returns:
            str: The generated answer from the pipeline.
        """
        return self.agenticRag.generate(question, stream)
